.permissions-section[data-v-94b20338]{margin-bottom:var(--space-xl)}.permissions-section h2[data-v-94b20338]{margin-bottom:var(--space-md)}.org-table a[data-v-94b20338]{text-decoration:none;color:var(--color-link)}.org-table a[data-v-94b20338]:hover{text-decoration:underline}.org-table .center[data-v-94b20338]{width:6rem;min-width:6rem}.org-table .role-names[data-v-94b20338]{max-width:200px;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.perm-name-cell[data-v-94b20338]{display:flex;flex-direction:column;gap:.3rem}.perm-title[data-v-94b20338]{font-weight:600;color:var(--color-heading)}.perm-id-info[data-v-94b20338]{font-size:.8rem;color:var(--color-text-muted);display:flex;align-items:center;gap:.5rem;flex-wrap:wrap}.perm-domain[data-v-94b20338]{color:var(--color-text-muted);font-size:.9rem}.perm-matrix-grid .org-head[data-v-94b20338]{display:flex;align-items:flex-end;justify-content:center}.perm-matrix-grid .org-head span[data-v-94b20338]{writing-mode:vertical-rl;transform:rotate(180deg);font-size:.65rem}.display-text[data-v-94b20338]{margin-right:var(--space-xs)}.edit-display-btn[data-v-94b20338]{padding:.1rem .2rem;font-size:.8rem}.edit-org-btn[data-v-94b20338]{padding:.1rem .2rem;font-size:.8rem;margin-left:var(--space-xs)}.perm-actions[data-v-94b20338]{text-align:center}.card.surface[data-v-1b84ed5d]{padding:var(--space-lg)}.org-title[data-v-1b84ed5d]{display:flex;align-items:center;gap:var(--space-sm);margin-bottom:var(--space-lg)}.org-name[data-v-1b84ed5d]{font-size:1.5rem;font-weight:600;color:var(--color-heading)}.perm-matrix-grid .role-head[data-v-1b84ed5d]{display:flex;align-items:flex-end;justify-content:center}.perm-matrix-grid .role-head span[data-v-1b84ed5d]{writing-mode:vertical-rl;transform:rotate(180deg);font-size:.65rem}.perm-matrix-grid .add-role-head[data-v-1b84ed5d]{cursor:pointer}.roles-grid[data-v-1b84ed5d]{display:flex;gap:var(--space-lg);margin-top:var(--space-lg)}.role-column[data-v-1b84ed5d]{flex:1;min-width:200px;border:1px solid var(--color-border);border-radius:var(--radius-md);padding:var(--space-md)}.role-header[data-v-1b84ed5d]{display:flex;justify-content:space-between;align-items:center;margin-bottom:var(--space-md)}.role-name[data-v-1b84ed5d]{display:flex;align-items:center;gap:var(--space-xs);font-size:1.1rem;color:var(--color-heading)}.role-actions[data-v-1b84ed5d]{display:flex;gap:var(--space-xs)}.plus-btn[data-v-1b84ed5d]{background:none;color:var(--color-accent);border:none;border-radius:var(--radius-sm);padding:.25rem .45rem;font-size:1.1rem;cursor:pointer}.plus-btn[data-v-1b84ed5d]:hover{background:#2563eb2e}.user-list[data-v-1b84ed5d]{list-style:none;padding:0;margin:0;display:flex;flex-direction:column;gap:var(--space-xs)}.user-chip[data-v-1b84ed5d]{background:var(--color-surface);border:1px solid var(--color-border);border-radius:var(--radius-md);padding:.45rem .6rem;display:flex;justify-content:space-between;gap:var(--space-sm);cursor:grab}.user-chip[data-v-1b84ed5d]:focus{outline:2px solid var(--color-accent);outline-offset:1px}.user-chip .meta[data-v-1b84ed5d]{font-size:.7rem;color:var(--color-text-muted)}.empty-role[data-v-1b84ed5d]{border:1px dashed var(--color-border-strong);border-radius:var(--radius-md);padding:var(--space-sm);display:flex;flex-direction:column;gap:var(--space-xs);align-items:flex-start}.empty-text[data-v-1b84ed5d]{margin:0}@media(max-width:720px){.roles-grid[data-v-1b84ed5d]{flex-direction:column}}.user-detail[data-v-01294e6a]{display:flex;flex-direction:column;gap:var(--space-lg)}.admin-actions[data-v-01294e6a]{display:flex;gap:.5rem}.ancillary-actions[data-v-01294e6a]{margin-top:-.5rem}.optional[data-v-f509d089]{font-weight:400;color:var(--color-text-muted);font-size:.85em}.view-admin[data-v-eff2f530]{padding-bottom:var(--space-3xl)}.admin-section[data-v-eff2f530]{margin-top:var(--space-xl)}.admin-section-body[data-v-eff2f530],.admin-panels[data-v-eff2f530]{display:flex;flex-direction:column;gap:var(--space-xl)}
