.reset-container[data-v-49fe4c15]{max-width:560px;margin:0 auto;width:100%}.reset-header[data-v-49fe4c15]{text-align:center}.section-body[data-v-49fe4c15]{gap:1.25rem}.name-edit span[data-v-49fe4c15]{color:var(--color-text-muted);font-size:.9rem}
