import{o as d,b as u,y as s,x as m,p as o,a8 as l,J as p,K as h}from"./_plugin-vue_export-helper-CX32kvNK.js";import{R as f}from"./RestrictedAuth-CyES_pxX.js";import{g as a}from"./helpers-DzjFIx78.js";import"./pow-DUr-T9XX.js";const w={__name:"RestrictedForward",setup(_){const n=o(()=>l()),r=o(()=>{const e=document.documentElement.getAttribute("data-mode");return e==="reauth"?"reauth":e==="forbidden"?"forbidden":"login"});function i(){location.reload()}function c(){const t=n.value||"/auth/";window.location.pathname!==t&&history.replaceState(null,"",t),window.location.href=t}return d(()=>{window.addEventListener("keydown",t=>{t.key==="Escape"&&a()})}),(t,e)=>(u(),s(f,{mode:r.value,onAuthenticated:i,onBack:m(a),onHome:c},null,8,["mode","onBack"]))}};p(w).mount("#app");h();
